<?php

use \Medoo\Medoo;

/*
 * This file is part of Source.
 *
 * (c) Ging - 2018
 *
 * file that was distributed with this source code.
 */

/**
 * This is a template core.
 *
 * @final since version 2.0
 *
 * @author Ging <gingdz11012001@gmail.com>
 */

class Twig_Core {

	public function __construct() {
		
		define('ROOT', $_SERVER['DOCUMENT_ROOT']);
		
		$this->db= new Medoo([
		       'database_type' => 'sqlite',
		       'database_file' => 'custom_data/data.sqlite'
		    ]);
	}

	public function get_data($key = null, $per_page = null, $page = null, $order = null) {
		if (!$key) return 'There is not key in get_data()';
		if (!$order) $order = 'time';
		if (!$per_page) $per_page = 10;
		if (!$page) $page = 0;
		$per_page = abs(intval($per_page));
		$page = abs(intval($page));
		$result = $this->db->select('custom_data', '*', ['key' => $key, 'ORDER' => [$order => 'DESC'], 'LIMIT' => [$page, $per_page]]);
		if ($result) {
	       return $result;
		}
	}

	public function get_data_by_id($key = null, $id = null) {
		if (!$key || !$id) return 'There is not key or id in get_data_by_id()';
		$id = abs(intval($id));
		$result = $this->db->select('custom_data', '*', ['id' => $id, 'key' => $key, 'LIMIT' => 1]);
		if ($result) {
			return $result[0];
		}
	}

	public function save_data($key = null, $data = null) {
		if (!$key || !$data) return 'There is not key or data in save_data()';
		$this->db->insert('custom_data', ['key' => $key, 'data' => $data, 'time' => time()]);
		return $this->db->id;
	}

	public function get_data_count($key = null) {
		if (!$key) return 'There is not key in get_data_count()';
		$result= $this->db->count('custom_data', ['key' => $key]);
		return $result;
	}
	
	public function delete_data_by_id($key = null, $id = null) {
		if (!$key || !$id) return 'There is not key or id in delete_data_by_id()';
		$id = abs(intval($id));
		$result = $this->db->count('custom_data', ['id' => $id, 'key' => $key, 'LIMIT' => 1]);
		if ($result == 1) {
			return $this->db->delete('custom_data', ['id' => $id], ['LIMIT' => 1]);
		}
	}
	
	public function update_data_by_id($key = null, $id = null, $data = null) {
		if (!$key || !$id) return 'There is not key or id in update_data_by_id()';
		$id = abs(intval($id));
		$result = $this->db->count('custom_data', ['id' => $id, 'key' => $key, 'LIMIT' => 1]);
		if ($result == 1) {
			$this->db->update('custom_data', ['data' => $data], ['id' => $id]);
		}
	}

	public function request_method()
	{
		return $_SERVER['REQUEST_METHOD'];
	}

	public function ip()
	{
		return $_SERVER['REMOTE_ADDR'];
	}
	
	public function get_uri_segments() {
		$uri_path = trim($_SERVER['REQUEST_URI'], '/');
		if(isset($_GET)) {
                $uri_path = explode('?',$uri_path);
                $uri_path = $uri_path[0];
            }
		$uri_segments = explode('/', $uri_path);
		return $uri_segments;
	}
	public function redirect($route) {
		return header('Location: ' . $route);
	}

	public function get_post($string) {
		return $_POST[$string];
	}

	public function get_get($string) {
		return $_GET[$string];
	}

	public function user_agent() {
		return $_SERVER['HTTP_USER_AGENT'];
	}

	public function set_cookie($name, $value) {
		return setcookie($name, $value, time() + 3600 * 24 * 365);
	}

	public function delete_cookie($name) {
		return setcookie($name, '', time() - 3600 * 24 * 365);
	}

	public function get_cookie($name) {
		return $_COOKIE[$name];
	}

	public function json_decode($string) {
		return json_decode($string, true);
	}

	public function format($name) {
		$var = explode('.', $name);
		if (count($var) < 2) {
			$ext = 'html';
		} else {
			$ext = array_pop($var);
		}

		$mime = array(
			'htm' => 'text/html',
			'html' => 'text/html',
			'php' => 'text/html',
			'txt' => 'text/plain',
			'css' => 'text/css',
			'js' => 'application/javascript',
			'json' => 'application/json',
			'xml' => 'application/xml',
			'rss' => 'application/rss+xml',
			'png' => 'image/png',
			'jpg' => 'image/jpeg',
			'jpeg' => 'image/jpeg',
			'gif' => 'image/gif',
			'ico' => 'image/x-icon'
		);
		
		if ($mime[$ext]) {
			return $mime[$ext];
		} else {
			return 'application/octet-stream';
		}
	}

	public function start() {
		
		$filter = array(
			'json_decode',
			'truncate',
			'slugify_url'
		);
		$functions = array(
			'ip',
			'get_uri_segments',
			'get_post',
			'get_get',
			'get_cookie',
			'set_cookie',
			'delete_cookie',
			'request_method',
			'user_agent',
			'redirect',
			'get_data',
			'save_data',
			'get_data_by_id',
			'update_data_by_id',
			'get_data_count',
			'delete_data_by_id'
		);
		
		$tmp = 'template';
		
		$url = $this->get_uri_segments();
		
		$route = $url[0];

		if ($route == '') {
			$route = 'index';
		} else {
			
			if (preg_match('/.twig/is', $route)) die('This file is protected.');
			
			if (!file_exists($tmp . '/' . $route)) {
				if (!file_exists($tmp . '/_404')) {
					die('Page not found.');
				} else {
					$route = '_404';
				}
			} else {
				
				if ($this->format($route) != 'text/html') {
			       header('Content-Type: ' . $this->format($route));
			       readfile($tmp . '/' . $route);
			       exit;
		        }
		
		   }
		
        }

		$loader = new Twig_Loader_Filesystem($tmp);
		$twig = new Twig_Environment($loader);
		foreach($functions as $function) {
			$data = new Twig_SimpleFunction($function, array(
				$this,
                $function
			));
			$twig->addFunction($data);
		}

		foreach($filter as $filter) {
			$data = new Twig_SimpleFilter($filter, array(
				$this,
				$filter
			));
			
			$twig->addFilter($data);
		}

		echo $twig->render($route);
	}
}